@@ -84,6 +84,7 @@
 #include "config.h"
 #endif
 
+#include "gstgdkpixbufelements.h"
 #include "gstgdkpixbufsink.h"
 
 #include <gst/video/video.h>
@@ -101,6 +102,8 @@ enum
 
 
 G_DEFINE_TYPE (GstGdkPixbufSink, gst_gdk_pixbuf_sink, GST_TYPE_VIDEO_SINK);
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (gdkpixbufsink, "gdkpixbufsink",
+    GST_RANK_NONE, GST_TYPE_GDK_PIXBUF_SINK, gdk_pixbuf_element_init (plugin));
 
 static void gst_gdk_pixbuf_sink_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
@@ -264,7 +267,7 @@ gst_gdk_pixbuf_sink_pixbuf_destroy_notify (guchar * pixels,
 {
   gst_video_frame_unmap (frame);
   gst_buffer_unref (frame->buffer);
-  g_slice_free (GstVideoFrame, frame);
+  g_free (frame);
 }
 
 static GdkPixbuf *
@@ -278,7 +281,7 @@ gst_gdk_pixbuf_sink_get_pixbuf_from_buffer (GstGdkPixbufSink * sink,
   g_return_val_if_fail (GST_VIDEO_SINK_WIDTH (sink) > 0, NULL);
   g_return_val_if_fail (GST_VIDEO_SINK_HEIGHT (sink) > 0, NULL);
 
-  frame = g_slice_new0 (GstVideoFrame);
+  frame = g_new0 (GstVideoFrame, 1);
   gst_video_frame_map (frame, &sink->info, buf, GST_MAP_READ);
 
   bytes_per_pixel = (sink->has_alpha) ? 4 : 3;
