@@ -54,11 +54,14 @@ G_BEGIN_DECLS
 #define FOURCC_FMP4     GST_MAKE_FOURCC('F','M','P','4')
 #define FOURCC_H264     GST_MAKE_FOURCC('H','2','6','4')
 #define FOURCC_H265     GST_MAKE_FOURCC('H','2','6','5')
+#define FOURCC_H266     GST_MAKE_FOURCC('H','2','6','6')
+#define FOURCC_LAGS     GST_MAKE_FOURCC('L','A','G','S')
 #define FOURCC_MAC3     GST_MAKE_FOURCC('M','A','C','3')
 #define FOURCC_MAC6     GST_MAKE_FOURCC('M','A','C','6')
 #define FOURCC_MP4V     GST_MAKE_FOURCC('M','P','4','V')
 #define FOURCC_PICT     GST_MAKE_FOURCC('P','I','C','T')
 #define FOURCC_QDM2     GST_MAKE_FOURCC('Q','D','M','2')
+#define FOURCC_QDMC     GST_MAKE_FOURCC('Q','D','M','C')
 #define FOURCC_SVQ3     GST_MAKE_FOURCC('S','V','Q','3')
 #define FOURCC_VP31     GST_MAKE_FOURCC('V','P','3','1')
 #define FOURCC_VP80     GST_MAKE_FOURCC('V','P','8','0')
@@ -115,6 +118,7 @@ G_BEGIN_DECLS
 #define FOURCC_clcp     GST_MAKE_FOURCC('c','l','c','p')
 #define FOURCC_clip     GST_MAKE_FOURCC('c','l','i','p')
 #define FOURCC_cmov     GST_MAKE_FOURCC('c','m','o','v')
+#define FOURCC_cmpd     GST_MAKE_FOURCC('c','m','p','d')
 #define FOURCC_cmvd     GST_MAKE_FOURCC('c','m','v','d')
 #define FOURCC_co64     GST_MAKE_FOURCC('c','o','6','4')
 #define FOURCC_covr     GST_MAKE_FOURCC('c','o','v','r')
@@ -210,6 +214,8 @@ G_BEGIN_DECLS
 #define FOURCC_dops     GST_MAKE_FOURCC('d','O','p','s')
 #define FOURCC_pasp     GST_MAKE_FOURCC('p','a','s','p')
 #define FOURCC_colr     GST_MAKE_FOURCC('c','o','l','r')
+#define FOURCC_clli     GST_MAKE_FOURCC('c','l','l','i')
+#define FOURCC_mdcv     GST_MAKE_FOURCC('m','d','c','v')
 #define FOURCC_clap     GST_MAKE_FOURCC('c','l','a','p')
 #define FOURCC_tapt     GST_MAKE_FOURCC('t','a','p','t')
 #define FOURCC_clef     GST_MAKE_FOURCC('c','l','e','f')
@@ -279,6 +285,8 @@ G_BEGIN_DECLS
 #define FOURCC_tx3g     GST_MAKE_FOURCC('t','x','3','g')
 #define FOURCC_udta     GST_MAKE_FOURCC('u','d','t','a')
 #define FOURCC_ulaw     GST_MAKE_FOURCC('u','l','a','w')
+#define FOURCC_uncC     GST_MAKE_FOURCC('u','n','c','C')
+#define FOURCC_uncv     GST_MAKE_FOURCC('u','n','c','v')
 #define FOURCC_url_     GST_MAKE_FOURCC('u','r','l',' ')
 #define FOURCC_uuid     GST_MAKE_FOURCC('u','u','i','d')
 #define FOURCC_v210     GST_MAKE_FOURCC('v','2','1','0')
@@ -292,12 +300,20 @@ G_BEGIN_DECLS
 #define FOURCC_vttc     GST_MAKE_FOURCC('v','t','t','c')
 #define FOURCC_vttC     GST_MAKE_FOURCC('v','t','t','C')
 #define FOURCC_vtte     GST_MAKE_FOURCC('v','t','t','e')
+#define FOURCC_vvc1     GST_MAKE_FOURCC('v','v','c','1')
+#define FOURCC_vvcC     GST_MAKE_FOURCC('v','v','c','C')
+#define FOURCC_vvi1     GST_MAKE_FOURCC('v','v','i','1')
 #define FOURCC_xvid     GST_MAKE_FOURCC('x','v','i','d')
 #define FOURCC_wave     GST_MAKE_FOURCC('w','a','v','e')
 #define FOURCC_wide     GST_MAKE_FOURCC('w','i','d','e')
 #define FOURCC_wvtt     GST_MAKE_FOURCC('w','v','t','t')
 #define FOURCC_zlib     GST_MAKE_FOURCC('z','l','i','b')
 #define FOURCC_lpcm     GST_MAKE_FOURCC('l','p','c','m')
+#define FOURCC_ipcm     GST_MAKE_FOURCC('i','p','c','m')
+#define FOURCC_fpcm     GST_MAKE_FOURCC('f','p','c','m')
+#define FOURCC_pcmC     GST_MAKE_FOURCC('p','c','m','C')
+#define FOURCC_chnl     GST_MAKE_FOURCC('c','h','n','l')
+#define FOURCC_chan     GST_MAKE_FOURCC('c','h','a','n')
 #define FOURCC_av01     GST_MAKE_FOURCC('a','v','0','1')
 #define FOURCC_av1C     GST_MAKE_FOURCC('a','v','1','C')
 #define FOURCC_av1f     GST_MAKE_FOURCC('a','v','1','f')
@@ -448,6 +464,15 @@ G_BEGIN_DECLS
 /* ONVIF Export File Format */
 #define FOURCC_cstb     GST_MAKE_FOURCC('c','s','t','b')
 
+/* Hap video codec */
+#define FOURCC_Hap1     GST_MAKE_FOURCC('H','a','p','1')
+#define FOURCC_Hap5     GST_MAKE_FOURCC('H','a','p','5')
+#define FOURCC_HapY     GST_MAKE_FOURCC('H','a','p','Y')
+#define FOURCC_HapM     GST_MAKE_FOURCC('H','a','p','M')
+#define FOURCC_HapA     GST_MAKE_FOURCC('H','a','p','A')
+#define FOURCC_Hap7     GST_MAKE_FOURCC('H','a','p','7')
+#define FOURCC_HapH     GST_MAKE_FOURCC('H','a','p','H')
+
 G_END_DECLS
 
 #endif /* __FOURCC_H__ */
