@@ -44,47 +44,62 @@ G_BEGIN_DECLS
 typedef struct _GstUDPSrc GstUDPSrc;
 typedef struct _GstUDPSrcClass GstUDPSrcClass;
 
+
+/**
+ * GstSocketTimestampMode:
+ * @GST_SOCKET_TIMESTAMP_MODE_DISABLED: Disable additional timestamps
+ * @GST_SOCKET_TIMESTAMP_MODE_REALTIME: Timestamp with realtime clock (nsec
+ *      resolution, may not be monotonic)
+ *
+ * Since: 1.20
+ */
+typedef enum
+{
+  GST_SOCKET_TIMESTAMP_MODE_DISABLED = 0,
+  GST_SOCKET_TIMESTAMP_MODE_REALTIME
+} GstSocketTimestampMode;
+
 struct _GstUDPSrc {
   GstPushSrc parent;
 
+  /* our sockets */
+  GSocket   *used_socket;	/* hot */
+  GInetSocketAddress *addr;	/* hot */
+  GList     *source_addrs;
+
+  GCancellable *cancellable;	/* hot */
+
   /* properties */
+  gint       skip_first_bytes;	/* hot */
+  guint64    timeout;	/* hot */
+  gboolean   retrieve_sender_address;	/* hot */
   gchar     *address;
   gint       port;
   gchar     *multi_iface;
-  gint       ttl;
   GstCaps   *caps;
   gint       buffer_size;
-  guint64    timeout;
-  gint       skip_first_bytes;
   GSocket   *socket;
   gboolean   close_socket;
   gboolean   auto_multicast;
   gboolean   reuse;
   gboolean   loop;
-  gboolean   retrieve_sender_address;
+  GstSocketTimestampMode socket_timestamp_mode;
+  gchar     *multicast_source;
 
   /* stats */
   guint      max_size;
 
-  /* our sockets */
-  GSocket   *used_socket;
-  GInetSocketAddress *addr;
   gboolean   external_socket;
-
   gboolean   made_cancel_fd;
-  GCancellable *cancellable;
 
-  /* memory management */
-  GstAllocator *allocator;
-  GstAllocationParams params;
+  /* Initial size of buffers in the buffer pool */
+  guint mtu;
 
-  GstMemory   *mem;
-  GstMapInfo   map;
-  GstMemory   *mem_max;
-  GstMapInfo   map_max;
-  GInputVector vec[2];
+  /* Extra memory for buffers with a size superior to max_packet_size */
+  GstMemory *extra_mem;
 
   gchar     *uri;
+  GPtrArray *source_list;
 };
 
 struct _GstUDPSrcClass {
