@@ -29,6 +29,7 @@
 
 G_BEGIN_DECLS
 
+#define GST_TYPE_V4L2_MEMORY_GROUP              (gst_v4l2_memory_group_get_type())
 #define GST_TYPE_V4L2_ALLOCATOR                 (gst_v4l2_allocator_get_type())
 #define GST_IS_V4L2_ALLOCATOR(obj)              (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GST_TYPE_V4L2_ALLOCATOR))
 #define GST_IS_V4L2_ALLOCATOR_CLASS(klass)      (G_TYPE_CHECK_CLASS_TYPE ((klass), GST_TYPE_V4L2_ALLOCATOR))
@@ -41,6 +42,10 @@ G_BEGIN_DECLS
         (GST_OBJECT_FLAG_IS_SET (obj, GST_V4L2_ALLOCATOR_FLAG_ ## type ## _REQBUFS))
 #define GST_V4L2_ALLOCATOR_CAN_ALLOCATE(obj,type) \
         (GST_OBJECT_FLAG_IS_SET (obj, GST_V4L2_ALLOCATOR_FLAG_ ## type ## _CREATE_BUFS))
+#define GST_V4L2_ALLOCATOR_CAN_ORPHAN_BUFS(obj) \
+        (GST_OBJECT_FLAG_IS_SET (obj, GST_V4L2_ALLOCATOR_FLAG_SUPPORTS_ORPHANED_BUFS))
+#define GST_V4L2_ALLOCATOR_IS_ORPHANED(obj) \
+        (GST_OBJECT_FLAG_IS_SET (obj, GST_V4L2_ALLOCATOR_FLAG_ORPHANED))
 
 #define GST_V4L2_MEMORY_QUARK gst_v4l2_memory_quark ()
 
@@ -50,7 +55,6 @@ typedef struct _GstV4l2MemoryGroup GstV4l2MemoryGroup;
 typedef struct _GstV4l2Memory GstV4l2Memory;
 typedef enum _GstV4l2Capabilities GstV4l2Capabilities;
 typedef enum _GstV4l2Return GstV4l2Return;
-typedef struct _GstV4l2Object GstV4l2Object;
 
 enum _GstV4l2AllocatorFlags
 {
@@ -60,6 +64,8 @@ enum _GstV4l2AllocatorFlags
   GST_V4L2_ALLOCATOR_FLAG_USERPTR_CREATE_BUFS = (GST_ALLOCATOR_FLAG_LAST << 3),
   GST_V4L2_ALLOCATOR_FLAG_DMABUF_REQBUFS      = (GST_ALLOCATOR_FLAG_LAST << 4),
   GST_V4L2_ALLOCATOR_FLAG_DMABUF_CREATE_BUFS  = (GST_ALLOCATOR_FLAG_LAST << 5),
+  GST_V4L2_ALLOCATOR_FLAG_SUPPORTS_ORPHANED_BUFS = (GST_ALLOCATOR_FLAG_LAST << 6),
+  GST_V4L2_ALLOCATOR_FLAG_ORPHANED            = (GST_ALLOCATOR_FLAG_LAST << 7),
 };
 
 enum _GstV4l2Return
@@ -106,6 +112,8 @@ struct _GstV4l2AllocatorClass {
   GstAllocatorClass parent_class;
 };
 
+GType gst_v4l2_memory_group_get_type(void);
+
 GType gst_v4l2_allocator_get_type(void);
 
 gboolean             gst_is_v4l2_memory                (GstMemory * mem);
@@ -123,6 +131,8 @@ guint                gst_v4l2_allocator_start          (GstV4l2Allocator * alloc
 
 GstV4l2Return        gst_v4l2_allocator_stop           (GstV4l2Allocator * allocator);
 
+gboolean             gst_v4l2_allocator_orphan         (GstV4l2Allocator * allocator);
+
 GstV4l2MemoryGroup*  gst_v4l2_allocator_alloc_mmap     (GstV4l2Allocator * allocator);
 
 GstV4l2MemoryGroup*  gst_v4l2_allocator_alloc_dmabuf   (GstV4l2Allocator * allocator,
