@@ -61,7 +61,7 @@
 
 #include "gstv4l2elements.h"
 #include "gstv4l2sink.h"
-#include "gst/gst-i18n-plugin.h"
+#include <glib/gi18n-lib.h>
 
 #include <string.h>
 
@@ -163,19 +163,21 @@ gst_v4l2sink_class_init (GstV4l2SinkClass * klass)
   g_object_class_install_property (gobject_class, PROP_CROP_TOP,
       g_param_spec_int ("crop-top", "Crop top",
           "The topmost (y) coordinate of the video crop; top left corner of image is 0,0",
-          0x80000000, 0x7fffffff, 0, G_PARAM_READWRITE));
+          0x80000000, 0x7fffffff, 0,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_CROP_LEFT,
       g_param_spec_int ("crop-left", "Crop left",
           "The leftmost (x) coordinate of the video crop; top left corner of image is 0,0",
-          0x80000000, 0x7fffffff, 0, G_PARAM_READWRITE));
+          0x80000000, 0x7fffffff, 0,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_CROP_WIDTH,
       g_param_spec_uint ("crop-width", "Crop width",
           "The width of the video crop; default is equal to negotiated image width",
-          0, 0xffffffff, 0, G_PARAM_READWRITE));
+          0, 0xffffffff, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_CROP_HEIGHT,
       g_param_spec_uint ("crop-height", "Crop height",
           "The height of the video crop; default is equal to negotiated image height",
-          0, 0xffffffff, 0, G_PARAM_READWRITE));
+          0, 0xffffffff, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   gst_element_class_set_static_metadata (element_class,
       "Video (video4linux2) Sink", "Sink/Video",
@@ -587,11 +589,11 @@ gst_v4l2sink_show_frame (GstVideoSink * vsink, GstBuffer * buf)
   GstFlowReturn ret;
   GstV4l2Sink *v4l2sink = GST_V4L2SINK (vsink);
   GstV4l2Object *obj = v4l2sink->v4l2object;
-  GstBufferPool *bpool = GST_BUFFER_POOL (obj->pool);
+  GstBufferPool *bpool = gst_v4l2_object_get_buffer_pool (obj);
 
   GST_DEBUG_OBJECT (v4l2sink, "render buffer: %p", buf);
 
-  if (G_UNLIKELY (obj->pool == NULL))
+  if (G_UNLIKELY (bpool == NULL))
     goto not_negotiated;
 
   if (G_UNLIKELY (!gst_buffer_pool_is_active (bpool))) {
@@ -611,7 +613,7 @@ gst_v4l2sink_show_frame (GstVideoSink * vsink, GstBuffer * buf)
 
   gst_buffer_ref (buf);
 again:
-  ret = gst_v4l2_buffer_pool_process (GST_V4L2_BUFFER_POOL_CAST (obj->pool),
+  ret = gst_v4l2_buffer_pool_process (GST_V4L2_BUFFER_POOL_CAST (bpool),
       &buf, NULL);
   if (ret == GST_FLOW_FLUSHING) {
     ret = gst_base_sink_wait_preroll (GST_BASE_SINK (vsink));
@@ -619,6 +621,8 @@ again:
       goto again;
   }
   gst_buffer_unref (buf);
+  if (bpool)
+    gst_object_unref (bpool);
 
   return ret;
 
@@ -633,6 +637,8 @@ activate_failed:
     GST_ELEMENT_ERROR (v4l2sink, RESOURCE, SETTINGS,
         (_("Failed to allocated required memory.")),
         ("Buffer pool activation failed"));
+    if (bpool)
+      gst_object_unref (bpool);
     return GST_FLOW_ERROR;
   }
 }
