@@ -9,7 +9,7 @@ AUDIO_CAPS="application/x-rtp,media=(string)audio,clock-rate=(int)8000,encoding-
 #DEST=192.168.1.126
 DEST=localhost
 
-VIDEO_DEC="rtph263pdepay ! ffdec_h263"
+VIDEO_DEC="rtph263pdepay ! avdec_h263"
 AUDIO_DEC="rtppcmadepay ! alawdec"
 
 VIDEO_SINK="videoconvert ! autovideosink"
@@ -17,7 +17,7 @@ AUDIO_SINK="audioconvert ! audioresample ! autoaudiosink"
 
 LATENCY=100
 
-gst-launch-1.0 -v gstrtpbin name=rtpbin latency=$LATENCY                                    \
+gst-launch-1.0 -v rtpbin name=rtpbin latency=$LATENCY                                    \
            udpsrc caps=$VIDEO_CAPS port=5000 ! rtpbin.recv_rtp_sink_0                   \
 	         rtpbin. ! $VIDEO_DEC ! $VIDEO_SINK                                     \
            udpsrc port=5001 ! rtpbin.recv_rtcp_sink_0                                   \
